@@ -91,9 +91,15 @@ def test_empty_scalar_reduction_select(constructor: Constructor) -> None:
     assert_equal_data(result, expected)
 
 
-def test_empty_scalar_reduction_with_columns(constructor: Constructor) -> None:
+def test_empty_scalar_reduction_with_columns(
+    constructor: Constructor, request: pytest.FixtureRequest
+) -> None:
     if "duckdb" in str(constructor) and DUCKDB_VERSION < (1, 3):
         pytest.skip()
+    if any(
+        x in str(constructor) for x in ("duckdb", "sqlframe", "ibis")
+    ) and DUCKDB_VERSION >= (1, 4):
+        request.applymarker(pytest.mark.xfail)
     from itertools import chain
 
     data = {
