@@ -9,7 +9,7 @@ from narwhals._utils import Implementation
 from tests.utils import Constructor, assert_equal_data
 
 if TYPE_CHECKING:
-    from narwhals._namespace import EagerAllowed
+    from narwhals._typing import EagerAllowed, Polars
 
 
 def test_from_dict(eager_backend: EagerAllowed) -> None:
@@ -28,9 +28,7 @@ def test_from_dict_schema(eager_backend: EagerAllowed) -> None:
 
 
 @pytest.mark.parametrize("backend", [Implementation.POLARS, "polars"])
-def test_from_dict_without_backend(
-    constructor: Constructor, backend: EagerAllowed
-) -> None:
+def test_from_dict_without_backend(constructor: Constructor, backend: Polars) -> None:
     pytest.importorskip("polars")
 
     df = (
@@ -51,12 +49,12 @@ def test_from_dict_without_backend_invalid(constructor: Constructor) -> None:
 def test_from_dict_with_backend_invalid() -> None:
     pytest.importorskip("duckdb")
     with pytest.raises(ValueError, match="lazy-only"):
-        nw.DataFrame.from_dict({"c": [1, 2], "d": [5, 6]}, backend="duckdb")
+        nw.DataFrame.from_dict({"c": [1, 2], "d": [5, 6]}, backend="duckdb")  # type: ignore[arg-type]
 
 
 @pytest.mark.parametrize("backend", [Implementation.POLARS, "polars"])
 def test_from_dict_one_native_one_narwhals(
-    constructor: Constructor, backend: EagerAllowed
+    constructor: Constructor, backend: Polars
 ) -> None:
     pytest.importorskip("polars")
 
