@@ -29,6 +29,7 @@ import pytest
 
 import narwhals as nw
 from narwhals._utils import Version
+from tests.conftest import sqlframe_pyspark_lazy_constructor
 from tests.utils import Constructor, maybe_get_modin_df
 
 if TYPE_CHECKING:
@@ -258,10 +259,7 @@ def test_eager_only_lazy_dask(eager_only: Any, context: Any) -> None:
 
 def test_series_only_sqlframe() -> None:  # pragma: no cover
     pytest.importorskip("sqlframe")
-    from sqlframe.duckdb import DuckDBSession
-
-    session = DuckDBSession()
-    df = session.createDataFrame([*zip(*data.values())], schema=[*data.keys()])
+    df = sqlframe_pyspark_lazy_constructor(data)
 
     with pytest.raises(TypeError, match="Cannot only use `series_only`"):
         nw.from_native(df, series_only=True)  # pyright: ignore[reportArgumentType, reportCallIssue]
@@ -282,10 +280,7 @@ def test_series_only_sqlframe() -> None:  # pragma: no cover
 )
 def test_eager_only_sqlframe(eager_only: Any, context: Any) -> None:  # pragma: no cover
     pytest.importorskip("sqlframe")
-    from sqlframe.duckdb import DuckDBSession
-
-    session = DuckDBSession()
-    df = session.createDataFrame([*zip(*data.values())], schema=[*data.keys()])
+    df = sqlframe_pyspark_lazy_constructor(data)
 
     with context:
         res = nw.from_native(df, eager_only=eager_only)
@@ -301,7 +296,7 @@ def test_interchange_protocol_non_v1() -> None:
     result = nw.from_native(mockdf, pass_through=True)
     assert result is mockdf
     with pytest.raises(TypeError):
-        nw.from_native(mockdf)
+        nw.from_native(mockdf)  # type: ignore[call-overload]
 
 
 def test_from_native_strict_native_series() -> None:
